/*==================================================================
  ==================================================================
  ==	XMusic -- An Interactive Interface to Csound		  ==
  ==       Copyright (C) 1988 by George D. Drapeau                ==
  ==                  All Rights Reserved                         ==
  ==================================================================
  ==================================================================
*/
#include <X/Xlib.h>
#include <stdio.h>
#include <string.h>

#define USEBITMAPS
#include "defs.h"
#include "externs.h"


DrawHelpWin()							    /* This function draws the Help window */
{
  int	i;
  char	*sPtr;

  XClear(helpWin);						    /* Clear the window before drawing in it */
  for (i = 0, sPtr = helpLabel[0];				    /* Draw every line of text in the Help window */
       i<HELPSIZE;
       i++, sPtr = helpLabel[i])
    {
      XText(helpWin, 5,
	    (int)((helpFont->height * 1.5) * i + 5),
	    sPtr, strlen(sPtr),
	    helpFont->id, helpFG, helpBG);
    }								    /* end for i = 0... */
}								    /* end function DrawHelpWin */

DrawToolWin()
{
  XPixmapPut(toolWin,0,0,2,2,sine64_width,sine64_height,
	     sineMap,GXcopy,AllPlanes);
  XPixmapPut(toolWin,0,0,2,2+env64_height,
	     env64_width,env64_height,
	     envMap,GXcopy,AllPlanes);
  XPixmapPut(toolWin,0,0,2,2+2*sum_height,
	     sum_width,sum_height,
	     sumMap,GXcopy,AllPlanes);
  XPixmapPut(toolWin,0,0,2,2+3*amp64_height,
	     amp64_width,amp64_height,
	     ampMap,GXcopy,AllPlanes);
  XPixmapPut(toolWin,0,0,2,2+4*param64_height,
	     param64_width,param64_height,
	     paramMap,GXcopy,AllPlanes);
  XPixmapPut(toolWin,0,0,2,2+5*noise64_height,
	     noise64_width,noise64_height,
	     noiseMap,GXcopy,AllPlanes);
  XPixmapPut(toolWin,0,0,2,2+6*eq64_height,
	     eq64_width,eq64_height,
	     eqMap,GXcopy,AllPlanes);
}								    /* end function DrawToolWin */



/*############################################################################
  FUNCTION	DrawMusicWin
  DESCRIPTION:
  	This function is responsible for drawing the Music window and
	storing its contents in Pixmap form.  This function calls other
	functions to do the actual screen drawing.   After the Music window
	has been drawn, this function calculates the area of the window that
	will be saved in a Pixmap.  If the whole Music window is visible,
	that is, if none of the window extends beyond the bounds of the
	console, then the whole window can be saved in a Pixmap.  But in
	many cases, parts of the Music window will extend beyond the screen
	bounds.	
	The Pixmap is saved in order that some drawing operations may be
	done more quickly and efficiently.  The largest gain in drawing time
	can be obtained by making the Music window sensitive to ExposeRegion
	events, and then drawing only the region that was exposed instead of
	the whole window.
*/


DrawMusicWin()
{
  WindowInfo	wInfo,rInfo;
  Tool		*tPtr;
  
  if (musicPixmap != (Pixmap)0)					    /* If a Pixmap exists already, then... */
    XFreePixmap(musicPixmap);					    /* ...free up the storage used by it. */
  XClear(musicWin);						    /* Clear the Music window before drawing */
  XQueryWindow(musicWin,&wInfo);				    /* Get info about size and position of the Music window */
  XQueryWindow(RootWindow,&rInfo);				    /* Get info about size and position of the Root window */
  for (tPtr = toolList; tPtr; tPtr = tPtr->next)		    /* Go through the list of tools so far and... */
    {
      DrawOneTool(tPtr);					    /* ...draw them, one by one, in the Music Window */
    }
  for (tPtr = toolList; tPtr; tPtr = tPtr->next)		    /* Go through the list of tools so far and... */
    {
      DrawToolConnection(tPtr);					    /* ...connect them, one by one, in the Music Window */
    }
  topX = (wInfo.x);						    /* Set initial coordinates for size of the pixmap... */
  topY = (wInfo.y);						    /* ...area to save */
  botX = (wInfo.x + wInfo.width);
  botY = (wInfo.y + wInfo.height);
  
  if ((wInfo.x >= rInfo.x) && (botX < rInfo.width))		    /* Determine x coordinates of window area to save */
    {								    /* X-axis completely inside screen bounds */
      topX = 0;
      wid = wInfo.width;
    }
  if ((wInfo.x >= rInfo.x) && (botX >= rInfo.width))		    /* Left part of window inside screen bounds,... */
    {								    /* ...right part extends beyond screen bounds */
      topX = 0;
      wid = rInfo.width - borderWidth - wInfo.x;
    }
  if ((wInfo.x < rInfo.x) && (botX < rInfo.width))		    /* Left part of window outside of screen bounds,... */
    {								    /* ...right part inside screen bounds */
      topX = abs(wInfo.x);
      wid = wInfo.width - abs(wInfo.x);
    }
  if ((wInfo.x <rInfo.x) && (botX >= rInfo.width))		    /* Left part AND right part of window extend... */
    {								    /* ...beyond screen bounds */
      topX = abs(wInfo.x);
      wid = rInfo.width - borderWidth;				    /* Width is width of the screen */
    }								    /* At this point, the x coordinates have been determined */
  
  if ((wInfo.y >= rInfo.y) && (botY < rInfo.height))		    /* Determine y coordinates of window area to save */
    {								    /* Y-axis completely inside screen bounds */
      topY = 0;
      hgt = wInfo.height;
    }
  if ((wInfo.y >= rInfo.y) && (botY >= rInfo.height))		    /* Top part of window inside screen bounds,... */
    {								    /* ...bottom part extends beyond screen bounds */
      topY = 0;
      hgt = rInfo.height - borderWidth - wInfo.y;
    }
  if ((wInfo.y < rInfo.y) && (botY < rInfo.height))		    /* Top part of window outside of screen bounds,... */
    {								    /* ...bottom part inside screen bounds */
      topY = abs(wInfo.y);
      hgt = wInfo.height - abs(wInfo.y);
    }
  if ((wInfo.y <rInfo.y) && (botY >= rInfo.height))		    /* Top part AND bottom part of window extend... */
    {								    /* ...beyond screen bounds */
      topY = abs(wInfo.y);
      hgt = rInfo.height - borderWidth;				    /* Height is height of the screen */
    }								    /* At this point, the y coordinates have been determined */

musicPixmap = XPixmapSave(musicWin,(int)topX,(int)topY,		    /* Save the area determined in a Pixmap */
			  (int)wid,(int)hgt);
if (musicPixmap == (Pixmap) 0)
  {
    printf("Unable to save the Pixmap, for some reason... exiting.\n");
    exit(1);							    /* Exit with an error condition */
  }								    /* end if *myPixmap... */
}								    /* end function DrawMusicWin */




DrawOneTool(tool)
     Tool	*tool;
{
  if (tool->type == ParamTool)
    DrawParamTool(tool);
  else
    XPixmapPut(musicWin,0,0,
	       tool->x - (TOOLSIZE/2), tool->y - (TOOLSIZE/2),
	       TOOLSIZE,TOOLSIZE,tool->map,GXcopy,AllPlanes);
}								    /* end function DrawOneTool */



DrawPixmap(pixmap,event)
Pixmap pixmap;
XExposeRegionEvent *event;
{
XPixmapPut(event->window,					    /* Refresh the region exposed with this information: */
	   event->x-topX,event->y-topY,				    /* Starting location from the Pixmap */
	   event->x,event->y,					    /* Destination location on the window */
	   event->width,event->height,				    /* Area to be refreshed with this Pixmap */
	   pixmap,GXcopy,AllPlanes);
}								    /* end function DrawPixmap */



DrawIconWin()
{
  XPixmapPut(iconWin,0,0,0,0,mIcon_width,mIcon_height,
	     iconMap,GXcopy,AllPlanes);
}								    /* end function DrawIconWin */




DrawInputWindow(prompt)
     char	*prompt;
{
  Vertex	vlist[5];
  int		y;
  static char	*str2 = "Press Return or Click Left Button";
  static char	*str3 = "To Enter, Click Middle or Right";
  static char	*str4 = "Button To Cancel.";

  XClear(inputWin);
  y = 2;
  XText(inputWin,2,y,
	prompt, strlen(prompt),
	musicFont->id,musicFG,musicBG);
  y += musicFont->height + 2;
  XText(inputWin,2,y,
	str2, strlen(str2),
	musicFont->id,musicFG,musicBG);
  y += musicFont->height + 2;
  XText(inputWin,2,y,
	str3, strlen(str3),
	musicFont->id,musicFG,musicBG);
  y += musicFont->height + 2;
  XText(inputWin,2,y,
	str4, strlen(str4),
	musicFont->id,musicFG,musicBG);
  y += musicFont->height + 3;
  vlist[0].x = vlist[1].x = vlist[4].x = 2;
  vlist[0].y = vlist[3].y = vlist[4].y = y;
  vlist[2].x = vlist[3].x = musicFont->width * (MAXSTRLEN + 2);
  vlist[2].y = vlist[1].y = y + musicFont->height + 7;
  vlist[0].flags = vlist[1].flags = vlist[2].flags =
    vlist[3].flags = vlist[4].flags = 0;
  XDraw(inputWin, vlist, 5, 1, 1, musicFG, GXcopy, AllPlanes);
}								    /* end function DrawInputWindow */



DrawInputString()
{
  XPixSet(inputWin, 5, (4*musicFont->height) + 15,		    /* Clear the area where the string will be drawn */
	  musicFont->width * (MAXSTRLEN + 1),
	  musicFont->height,musicBG);
  XText(inputWin, 5, (4*musicFont->height) + 15,		    /* Draw the string */
	inputStr, inputStrLen, musicFont->id, musicFG, musicBG);
}								    /* end function DrawInputString */


DrawParamTool(tPtr)
     Tool	*tPtr;
{
  XText(musicWin,
	tPtr->x - ((strlen(tPtr->fName) * musicFont->width) / 2),
	tPtr->y,
	tPtr->fName, strlen(tPtr->fName),
	musicFont->id, musicFG, musicBG);
}								    /* end function DrawParamTool */


DrawToolConnection(tool)					    /* This function draws a line between this tool... */
     Tool	*tool;						    /* ...and the tools that serve as input to this one. */
{
  if (tool->type == SineTool)
    {
      DrawSineTool(tool);
      return;
    }
  if (tool->input1 != NULL)					    /* Is the first input being used? */
    XLine(musicWin, tool->x, tool->y - (TOOLSIZE/2),		    /* Yes, draw a connection between this tool and the... */
	  tool->input1->x, tool->input1->y + (TOOLSIZE/2),	    /* ...tool pointed to by input1  */
	  1,1,
	  musicFG, GXcopy, AllPlanes);
  if (tool->input2 != NULL)					    /* Is the second input being used? */
    XLine(musicWin, tool->x, tool->y - (TOOLSIZE/2),		    /* Yes, draw a connection between this tool and the... */
	  tool->input2->x, tool->input2->y + (TOOLSIZE/2),	    /* ...tool pointed to by input2 */
	  1,1,
	  musicFG, GXcopy, AllPlanes);
  if (tool->input3 != NULL)					    /* Is the third input being used? */
    XLine(musicWin, tool->x, tool->y - (TOOLSIZE/2),		    /* Yes, draw a connection between this tool and the... */
	  tool->input3->x, tool->input3->y + (TOOLSIZE/2),	    /* ...tool pointed to by input3 */
	  1,1,
	  musicFG, GXcopy, AllPlanes);
  if (tool->input4 != NULL)					    /* Is the fourth input being used? */
    XLine(musicWin, tool->x, tool->y - (TOOLSIZE/2),		    /* Yes, draw a connection between this tool and the... */
	  tool->input4->x, tool->input4->y + (TOOLSIZE/2),	    /* ...tool pointed to by input4 */
	  1,1,
	  musicFG, GXcopy, AllPlanes);
  if (tool->input5 != NULL)					    /* Is the fifth input being used? */
    XLine(musicWin, tool->x, tool->y - (TOOLSIZE/2),		    /* Yes, draw a connection between this tool and the... */
	  tool->input5->x, tool->input5->y + (TOOLSIZE/2),	    /* ...tool pointed to by input5 */
	  1,1,
	  musicFG, GXcopy, AllPlanes);
  if (tool->input6 != NULL)					    /* Is the sixth input being used? */
    XLine(musicWin, tool->x, tool->y - (TOOLSIZE/2),		    /* Yes, draw a connection between this tool and the... */
	  tool->input6->x, tool->input6->y + (TOOLSIZE/2),	    /* ...tool pointed to by input6 */
	  1,1,
	  musicFG, GXcopy, AllPlanes);
  if (tool->input7 != NULL)					    /* Is the seventh input being used? */
    XLine(musicWin, tool->x, tool->y - (TOOLSIZE/2),		    /* Yes, draw a connection between this tool and the... */
	  tool->input7->x, tool->input7->y + (TOOLSIZE/2),	    /* ...tool pointed to by input7 */
	  1,1,
	  musicFG, GXcopy, AllPlanes);
  if (tool->input8 != NULL)					    /* Is the eighth input being used? */
    XLine(musicWin, tool->x, tool->y - (TOOLSIZE/2),		    /* Yes, draw a connection between this tool and the... */
	  tool->input8->x, tool->input8->y + (TOOLSIZE/2),	    /* ...tool pointed to by input8 */
	  1,1,
	  musicFG, GXcopy, AllPlanes);
}								    /* end function DrawToolConnection */


DrawSineTool(tool)
     Tool	*tool;
{
  if (tool->input1 != NULL)					    /* Is the Amplitude input being used? */
    XLine(musicWin, tool->x - (TOOLSIZE/4),			    /* Yes, draw a connection between this tool and the... */
	  tool->y - (TOOLSIZE/2),				    /* ...tool pointed to by the Amplitude input.  */
	  tool->input1->x, tool->input1->y + (TOOLSIZE/2),
	  1,1,
	  musicFG, GXcopy, AllPlanes);
  if (tool->input2 != NULL)					    /* Is the Frequency input being used? */
    XLine(musicWin, tool->x + (TOOLSIZE/4),			    /* Yes, draw a connection between this tool and the... */
	  tool->y - (TOOLSIZE/2),				    /* ...tool pointed to by the Frequency input. */
	  tool->input2->x, tool->input2->y + (TOOLSIZE/2),
	  1,1,
	  musicFG, GXcopy, AllPlanes);
  if (tool->input3 != NULL)					    /* Is the third input being used? */
    XLine(musicWin, tool->x, tool->y - (TOOLSIZE/2),		    /* Yes, draw a connection between this tool and the... */
	  tool->input3->x, tool->input3->y + (TOOLSIZE/2),	    /* ...tool pointed to by input3 */
	  1,1,
	  musicFG, GXcopy, AllPlanes);
  if (tool->input4 != NULL)					    /* Is the fourth input being used? */
    XLine(musicWin, tool->x, tool->y - (TOOLSIZE/2),		    /* Yes, draw a connection between this tool and the... */
	  tool->input4->x, tool->input4->y + (TOOLSIZE/2),	    /* ...tool pointed to by input4 */
	  1,1,
	  musicFG, GXcopy, AllPlanes);
  if (tool->input5 != NULL)					    /* Is the fifth input being used? */
    XLine(musicWin, tool->x, tool->y - (TOOLSIZE/2),		    /* Yes, draw a connection between this tool and the... */
	  tool->input5->x, tool->input5->y + (TOOLSIZE/2),	    /* ...tool pointed to by input5 */
	  1,1,
	  musicFG, GXcopy, AllPlanes);
  if (tool->input6 != NULL)					    /* Is the sixth input being used? */
    XLine(musicWin, tool->x, tool->y - (TOOLSIZE/2),		    /* Yes, draw a connection between this tool and the... */
	  tool->input6->x, tool->input6->y + (TOOLSIZE/2),	    /* ...tool pointed to by input6 */
	  1,1,
	  musicFG, GXcopy, AllPlanes);
  if (tool->input7 != NULL)					    /* Is the seventh input being used? */
    XLine(musicWin, tool->x, tool->y - (TOOLSIZE/2),		    /* Yes, draw a connection between this tool and the... */
	  tool->input7->x, tool->input7->y + (TOOLSIZE/2),	    /* ...tool pointed to by input7 */
	  1,1,
	  musicFG, GXcopy, AllPlanes);
  if (tool->input8 != NULL)					    /* Is the eighth input being used? */
    XLine(musicWin, tool->x, tool->y - (TOOLSIZE/2),		    /* Yes, draw a connection between this tool and the... */
	  tool->input8->x, tool->input8->y + (TOOLSIZE/2),	    /* ...tool pointed to by input8 */
	  1,1,
	  musicFG, GXcopy, AllPlanes);
}								    /* end function DrawToolConnection */
