/*==================================================================
  ==================================================================
  ==	XMusic -- An Interactive Interface to Csound		  ==
  ==       Copyright (C) 1988 by George D. Drapeau                ==
  ==                  All Rights Reserved                         ==
  ==================================================================
  ==================================================================
*/
#include <X/Xlib.h>
#include <stdio.h>
#include <string.h>
#include "defs.h"
#include "externs.h"



SelectTool(event)
     XKeyOrButtonEvent	*event;
{
  int	whichTool;

  whichTool = (event->y+2) /64;
  return(whichTool);
}								    /* end function SelectTool */




MoveTool()
{
  Window	subW;
  int		x,y;
  XEvent	event;
  Tool		*tool, *GetToolFromCoords();

  XQueryMouse(musicWin,&x,&y,&subW);
  if ((tool = GetToolFromCoords(x,y)) == NULL)
      {
	printf("Not pointing to any tool.\n");
	return;
      }
  XGrabMouse(musicWin,tCursor,ButtonPressed);
  XNextEvent(&event);
  XQueryMouse(musicWin,&x,&y,&subW);
  tool->x = x;
  tool->y = y;
  XUngrabMouse();
  DrawMusicWin();
}								    /* end function MoveTool */



RemoveTool()
{
  Window	subW;
  int		x,y;
  XEvent	event;
  Tool		*tool, *GetToolFromCoords();

  XQueryMouse(musicWin,&x,&y,&subW);
  if ((tool = GetToolFromCoords(x,y)) == NULL)
      {
	printf("Not pointing to any tool.\n");
	return;
      }
  DeleteToolFromList(tool);
  DrawMusicWin();						    /* Refresh the diagram to reflect the deleted tool */
}								    /* end function RemoveTool */



Tool	*GetToolFromCoords(x,y)
     int	x,y;
{
  Tool	*tPtr;

  for (tPtr = toolList;  tPtr;  tPtr = tPtr->next)		    /* Examine every tool in the list */
    {
      if ((x > (tPtr->x - (TOOLSIZE/2))) &&			    /* Are the coordinates passed in inside the... */
	  (x < (tPtr->x + (TOOLSIZE/2))) &&			    /* ...area that would be occupied by this... */
	  (y > (tPtr->y - (TOOLSIZE/2))) &&			    /* ...tool? */
	  (y < (tPtr->y + (TOOLSIZE/2))))			    /* If so, return a pointer to the tool in which... */
	return(tPtr);						    /* ...the coordinates were found. */
    }
  return(NULL);							    /* No tool is close to the given coordinates */
}								    /* end function GetToolFromCoords */



Tool	*MakeParamTool(x,y)
     int	x,y;
{
  Tool		*tPtr, *AddToolToList();
  char		*malloc(), tmpName[32];
  int		sLen;

  sLen = EnterString("Enter Parameter Name:");			    /* Get the parameter name from the user */
  if (sLen > 0)							    /* Did the user actually enter a string name? */
    {
      tPtr = AddToolToList();					    /* Yes, make a new tool and return a pointer to it */
      sprintf(tmpName, "paramtool%d",tPtr->toolNum);		    /* Generate a new name for this tool */
      tPtr->name = malloc(strlen(tmpName));			    /* Allocate space to hold the tool name */
      strcpy(tPtr->name, tmpName);				    /* Assign the new name to this tool */
      tPtr->fName = malloc(sLen+1);				    /* Allocate space to hold the Csound function name */
      strncpy(tPtr->fName, inputStr, sLen);			    /* Assign the function name to this tool */
      tPtr->fName[inputStrLen] = '\0';				    /* Append null byte to string. */
      tPtr->x = x;
      tPtr->y = y;
      tPtr->type = ParamTool;
      return(tPtr);
    }								    /* end if */
  else
    return(NULL);						    /* No, return a null pointer */
}								    /* end function MakeParamTool */



Tool	*MakeEQTool(x,y)
     int	x,y;
{
  Tool		*tPtr, *AddToolToList();
  char		*malloc(), tmpName[32];
  int		sLen;

  sLen = EnterString("Enter An Equation:");			    /* Let the user enter an equation */
  if (sLen > 0)							    /* Did the user actually enter a string name? */
    {
      tPtr = AddToolToList();					    /* Yes, make a new tool and return a pointer to it */
      sprintf(tmpName, "eqtool%d",tPtr->toolNum);		    /* Generate a new name for this tool */
      tPtr->name = malloc(strlen(tmpName));			    /* Allocate space to hold the tool name */
      strcpy(tPtr->name, tmpName);				    /* Assign the new name to this tool */
      tPtr->fName = malloc(sLen+1);				    /* Allocate space to hold the Csound function name */
      strncpy(tPtr->fName, inputStr, sLen);			    /* Assign the function name to this tool */
      tPtr->fName[inputStrLen] = '\0';				    /* Append null byte to string. */
      tPtr->x = x;
      tPtr->y = y;
      tPtr->type = EQTool;
      return(tPtr);
    }								    /* end if */
  else
    return(NULL);						    /* No, return a null pointer */
}								    /* end function MakeEQTool */
